@@ -1,15 +1,24 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import hashlib
 import logging
 
-from django.contrib.auth.models import AnonymousUser
 from django.conf import settings
-
+from django.contrib.auth import models
 from keystoneclient import exceptions as keystone_exceptions
 
-from .utils import check_token_expiration
-from .utils import get_keystone_version
-from .utils import get_project_list
-from .utils import is_ans1_token
+from openstack_auth import utils
 
 
 LOG = logging.getLogger(__name__)
@@ -17,20 +26,25 @@ LOG = logging.getLogger(__name__)
 
 def set_session_from_user(request, user):
     request.session['token'] = user.token
-    if 'token_list' not in request.session:
-        request.session['token_list'] = []
-    token_tuple = (user.endpoint, user.token.id)
-    request.session['token_list'].append(token_tuple)
     request.session['user_id'] = user.id
     request.session['region_endpoint'] = user.endpoint
     request.session['services_region'] = user.services_region
+    # Update the user object cached in the request
+    request._cached_user = user
+    request.user = user
 
 
 def create_user_from_token(request, token, endpoint, services_region=None):
+    # if the region is provided, use that, otherwise use the preferred region
+    svc_region = services_region or \
+        utils.default_services_region(token.serviceCatalog, request)
     return User(id=token.user['id'],
                 token=token,
                 user=token.user['name'],
                 user_domain_id=token.user_domain_id,
+                # We need to consider already logged-in users with an old
+                # version of Token without user_domain_name.
+                user_domain_name=getattr(token, 'user_domain_name', None),
                 project_id=token.project['id'],
                 project_name=token.project['name'],
                 domain_id=token.domain['id'],
@@ -39,7 +53,7 @@ def create_user_from_token(request, token, endpoint, services_region=None):
                 service_catalog=token.serviceCatalog,
                 roles=token.roles,
                 endpoint=endpoint,
-                services_region=services_region)
+                services_region=svc_region)
 
 
 class Token(object):
@@ -56,11 +70,16 @@ class Token(object):
         user['name'] = auth_ref.username
         self.user = user
         self.user_domain_id = auth_ref.user_domain_id
+        self.user_domain_name = auth_ref.user_domain_name
 
-        #Token-related attributes
+        # Token-related attributes
         self.id = auth_ref.auth_token
-        if is_ans1_token(self.id):
-            self.id = hashlib.md5(self.id).hexdigest()
+        if len(self.id) > 64:
+            algorithm = getattr(settings, 'OPENSTACK_TOKEN_HASH_ALGORITHM',
+                                'md5')
+            hasher = hashlib.new(algorithm)
+            hasher.update(self.id)
+            self.id = hasher.hexdigest()
         self.expires = auth_ref.expires
 
         # Project-related attributes
@@ -81,13 +100,13 @@ class Token(object):
         else:
             self.roles = auth_ref.get('roles', [])
 
-        if get_keystone_version() < 3:
+        if utils.get_keystone_version() < 3:
             self.serviceCatalog = auth_ref.get('serviceCatalog', [])
         else:
             self.serviceCatalog = auth_ref.get('catalog', [])
 
 
-class User(AnonymousUser):
+class User(models.AnonymousUser):
     """A User class with some extra special sauce for Keystone.
 
     In addition to the standard Django user attributes, this class also has
@@ -138,6 +157,10 @@ class User(AnonymousUser):
 
         The domain id of the current user.
 
+    .. attribute:: user_domain_name
+
+        The domain name of the current user.
+
     .. attribute:: domain_id
 
         The id of the Keystone domain scoped for the current user/token.
@@ -146,20 +169,24 @@ class User(AnonymousUser):
     def __init__(self, id=None, token=None, user=None, tenant_id=None,
                  service_catalog=None, tenant_name=None, roles=None,
                  authorized_tenants=None, endpoint=None, enabled=False,
-                 services_region=None, user_domain_id=None, domain_id=None,
-                 domain_name=None, project_id=None, project_name=None):
+                 services_region=None, user_domain_id=None,
+                 user_domain_name=None, domain_id=None, domain_name=None,
+                 project_id=None, project_name=None):
         self.id = id
         self.pk = id
         self.token = token
         self.username = user
         self.user_domain_id = user_domain_id
+        self.user_domain_name = user_domain_name
         self.domain_id = domain_id
         self.domain_name = domain_name
         self.project_id = project_id or tenant_id
         self.project_name = project_name or tenant_name
         self.service_catalog = service_catalog
-        self._services_region = services_region or \
-                                    self.default_services_region()
+        self._services_region = (
+            services_region
+            or utils.default_services_region(service_catalog)
+        )
         self.roles = roles or []
         self.endpoint = endpoint
         self.enabled = enabled
@@ -175,24 +202,56 @@ class User(AnonymousUser):
     def __repr__(self):
         return "<%s: %s>" % (self.__class__.__name__, self.username)
 
-    def is_token_expired(self):
-        """
+    def is_token_expired(self, margin=None):
+        """Determine if the token is expired.
+
         Returns ``True`` if the token is expired, ``False`` if not, and
         ``None`` if there is no token set.
+
+        .. param:: margin
+
+           A security time margin in seconds before real expiration.
+           Will return ``True`` if the token expires in less than ``margin``
+           seconds of time.
+           A default margin can be set by the TOKEN_TIMEOUT_MARGIN in the
+           django settings.
+
         """
         if self.token is None:
             return None
-        return not check_token_expiration(self.token)
+        return not utils.is_token_valid(self.token, margin)
 
-    def is_authenticated(self):
-        """ Checks for a valid token that has not yet expired. """
-        return self.token is not None and check_token_expiration(self.token)
+    def is_authenticated(self, margin=None):
+        """Checks for a valid authentication.
+
+        .. param:: margin
+
+           A security time margin in seconds before end of authentication.
+           Will return ``False`` if authentication ends in less than ``margin``
+           seconds of time.
+           A default margin can be set by the TOKEN_TIMEOUT_MARGIN in the
+           django settings.
 
-    def is_anonymous(self):
         """
-        Returns ``True`` if the user is not authenticated,``False`` otherwise.
+        return (self.token is not None and
+                utils.is_token_valid(self.token, margin))
+
+    def is_anonymous(self, margin=None):
+        """Return if the user is not authenticated.
+
+        Returns ``True`` if not authenticated,``False`` otherwise.
+
+        .. param:: margin
+
+           A security time margin in seconds before end of an eventual
+           authentication.
+           Will return ``True`` even if authenticated but that authentication
+           ends in less than ``margin`` seconds of time.
+           A default margin can be set by the TOKEN_TIMEOUT_MARGIN in the
+           django settings.
+
         """
-        return not self.is_authenticated()
+        return not self.is_authenticated(margin)
 
     @property
     def is_active(self):
@@ -200,26 +259,33 @@ class User(AnonymousUser):
 
     @property
     def is_superuser(self):
+        """Evaluates whether this user has admin privileges.
+
+        Returns ``True`` or ``False``.
         """
-        Evaluates whether this user has admin privileges. Returns
-        ``True`` or ``False``.
-        """
-        return 'admin' in [role['name'].lower() for role in self.roles]
+        admin_roles = [role.lower() for role in getattr(
+            settings,
+            'OPENSTACK_KEYSTONE_ADMIN_ROLES',
+            ['admin'])]
+        user_roles = [role['name'].lower() for role in self.roles]
+        return True if set(admin_roles).intersection(user_roles) else False
 
     @property
     def authorized_tenants(self):
-        """ Returns a memoized list of tenants this user may access. """
+        """Returns a memoized list of tenants this user may access."""
         insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+        ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
 
         if self.is_authenticated() and self._authorized_tenants is None:
             endpoint = self.endpoint
             token = self.token
             try:
-                self._authorized_tenants = get_project_list(
+                self._authorized_tenants = utils.get_project_list(
                     user_id=self.id,
                     auth_url=endpoint,
                     token=token.id,
                     insecure=insecure,
+                    cacert=ca_cert,
                     debug=settings.DEBUG)
             except (keystone_exceptions.ClientException,
                     keystone_exceptions.AuthorizationFailure):
@@ -230,19 +296,6 @@ class User(AnonymousUser):
     def authorized_tenants(self, tenant_list):
         self._authorized_tenants = tenant_list
 
-    def default_services_region(self):
-        """
-        Returns the first endpoint region for first non-identity service
-        in the service catalog
-        """
-        if self.service_catalog:
-            for service in self.service_catalog:
-                if service['type'] == 'identity':
-                    continue
-                for endpoint in service['endpoints']:
-                    return endpoint['region']
-        return None
-
     @property
     def services_region(self):
         return self._services_region
@@ -253,9 +306,7 @@ class User(AnonymousUser):
 
     @property
     def available_services_regions(self):
-        """
-        Returns list of unique region name values found in service catalog
-        """
+        """Returns list of unique region name values in service catalog."""
         regions = []
         if self.service_catalog:
             for service in self.service_catalog:
@@ -277,10 +328,10 @@ class User(AnonymousUser):
     # Check for OR'd permission rules, check that user has one of the
     # required permission.
     def has_a_matching_perm(self, perm_list, obj=None):
-        """
-        Returns True if the user has one of the specified permissions. If
-        object is passed, it checks if the user has any of the required perms
-        for this object.
+        """Returns True if the user has one of the specified permissions.
+
+        If object is passed, it checks if the user has any of the required
+        perms for this object.
         """
         # If there are no permissions to check, just return true
         if not perm_list:
@@ -304,8 +355,8 @@ class User(AnonymousUser):
     #   ('openstack.roles.admin', ('openstack.roles.L3-support',
     #                              'openstack.roles.L2-support'),)
     def has_perms(self, perm_list, obj=None):
-        """
-        Returns True if the user has all of the specified permissions.
+        """Returns True if the user has all of the specified permissions.
+
         Tuples in the list will possess the required permissions if
         the user has a permissions matching one of the elements of
         that tuple
